@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
 // This file is part of CERTI
 //
@@ -18,7 +18,7 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: ObjectManagement.cc,v 3.14 2004/01/09 16:41:00 breholee Exp $
+// $Id: ObjectManagement.cc,v 3.17 2005/04/30 16:38:39 breholee Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -35,11 +35,12 @@
 
 using std::cout ;
 using std::endl ;
+using std::string ;
 
 namespace certi {
 namespace rtia {
 
-static pdCDebug D("RTIA_OM", "(RTIA OM) ");
+static pdCDebug D("RTIA_OM", __FILE__);
 
 ObjectManagement::ObjectManagement(Communications *GC,
                                    FederationManagement *GF,
@@ -179,6 +180,7 @@ ObjectManagement::sendInteraction(InteractionClassHandle theInteraction,
                                   UShort paramArraySize,
                                   FederationTime theTime,
                                   const char *theTag,
+				  RegionHandle region,
                                   TypeException &e)
 {
     NetworkMessage req, rep ;
@@ -193,17 +195,16 @@ ObjectManagement::sendInteraction(InteractionClassHandle theInteraction,
     req.type = NetworkMessage::SEND_INTERACTION ;
     req.interactionClass = theInteraction ;
     req.date = theTime ;
-
+    req.region = region ;
     req.federation = fm->_numero_federation ;
     req.federate = fm->federate ;
 
     req.handleArraySize = paramArraySize ;
 
-    for (int i=0 ; i<paramArraySize ; i++)
-        {
-            req.handleArray[i] = paramArray[i] ;
-            req.setValue(i, valueArray[i]);
-        }
+    for (int i=0 ; i<paramArraySize ; i++) {
+	req.handleArray[i] = paramArray[i] ;
+	req.setValue(i, valueArray[i]);
+    }
 
     strcpy(req.label, theTag);
 
@@ -299,7 +300,7 @@ ObjectManagement::removeObject(ObjectHandle,
                                TypeException &)
 {
     printf("ObjectManagement.cc: RemoveObject(2) not implemented.\n");
-    throw RTIinternalError();
+    throw RTIinternalError("");
 }
 
 // ----------------------------------------------------------------------------
@@ -450,7 +451,7 @@ ObjectManagement::provideAttributeValueUpdate(ObjectHandle,
 {
     printf("ObjectManagement.cc: "
            "provideAttributeValueUpdate not implemented\n");
-    throw RTIinternalError();
+    throw RTIinternalError("");
 }
 
 
@@ -462,7 +463,7 @@ void ObjectManagement::retract(EventRetractionHandle /*theHandle*/,
                                TypeException & /*e*/)
 {
     printf("GO.cc: retract not implemented.\n");
-    throw RTIinternalError();
+    throw RTIinternalError("");
 }
 
 
@@ -475,7 +476,7 @@ ObjectManagement::reflectRetraction(EventRetractionHandle,
                                     TypeException &)
 {
     printf("GO.cc: reflectRetraction not implemented.\n");
-    throw RTIinternalError();
+    throw RTIinternalError("");
 }
 
 // ----------------------------------------------------------------------------
@@ -489,7 +490,7 @@ ObjectManagement::getObjectClassHandle(const char *theName)
 
 // ----------------------------------------------------------------------------
 //! getObjectClassName.
-const char *
+string
 ObjectManagement::getObjectClassName(ObjectClassHandle theHandle)
 {
     return rootObject->ObjectClasses->getObjectClassName(theHandle);
@@ -579,4 +580,4 @@ ObjectManagement::getObjectClass(ObjectHandle object)
 
 }} // namespace certi/rtia
 
-// $Id: ObjectManagement.cc,v 3.14 2004/01/09 16:41:00 breholee Exp $
+// $Id: ObjectManagement.cc,v 3.17 2005/04/30 16:38:39 breholee Exp $
