@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
 // This file is part of CERTI
 //
@@ -18,7 +18,7 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: RTIA.cc,v 3.10 2003/06/27 17:26:28 breholee Exp $
+// $Id: RTIA.cc,v 3.11 2005/04/30 16:38:39 breholee Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -102,7 +102,7 @@ RTIA::execute()
             comm->readMessage(n, msg_tcp_udp, msg_un);
         }
         catch (NetworkSignal) {
-            fm->_fin_execution = RTI_TRUE ;
+            fm->_fin_execution = true ;
             n = 0 ;
             delete msg_un ;
             delete msg_tcp_udp ;
@@ -127,4 +127,4 @@ RTIA::execute()
 
 }} // namespace certi/rtia
 
-// $Id: RTIA.cc,v 3.10 2003/06/27 17:26:28 breholee Exp $
+// $Id: RTIA.cc,v 3.11 2005/04/30 16:38:39 breholee Exp $
