@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
 // This file is part of CERTI
 //
@@ -18,7 +18,7 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: RTIA_federate.cc,v 3.27 2003/11/13 10:52:20 breholee Exp $
+// $Id: RTIA_federate.cc,v 3.31 2005/04/30 16:38:39 breholee Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -123,7 +123,7 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
           if (fdd->is_open()) {
 	      int result = certi::fedparser::build(filename.c_str(),
 						   rootObject, true);
-	      if (result) throw ErrorReadingFED();
+	      if (result) throw ErrorReadingFED("");
           }
           else {
               if (XmlParser::exists()) {
@@ -136,7 +136,7 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
                       delete fdd ;
                       delete parser ;
                   }
-		  else throw CouldNotOpenFED();
+		  else throw CouldNotOpenFED("");
               }
           }
           break ;
@@ -314,6 +314,7 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
                                       req->handleArraySize,
                                       req->getFederationTime(),
                                       req->getTag(),
+				      req->getRegion(),
                                       e));
               free(ValueArray);
           } catch (Exception *e) {
@@ -548,7 +549,7 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
               "Receiving Message from Federate, type GetObjectClassName.");
 
         try {
-            rep.setName(om->getObjectClassName(req->getObjectClass()));
+            rep.setName(om->getObjectClassName(req->getObjectClass()).c_str());
         }
         catch (ObjectClassNotDefined) {
             rep.setException(e_ObjectClassNotDefined);
@@ -682,8 +683,19 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
 			     req->handleArray, req->handleArraySize, e);
 	break ;
 
+      case Message::DDM_REGISTER_OBJECT:
+	D[pdTrace] << "Receiving Message from Federate: Register with Region"
+		   << endl ;
+	rep.setObject(ddm->registerObject(req->getObjectClass(),
+					  req->getName(),
+					  req->handleArray,
+					  req->handleArraySize,
+					  req->getRegions(),
+					  e));
+	break ;
+
       case Message::DDM_UNASSOCIATE_REGION:
-	D[pdTrace] << "Receiving Message from Federate: Associate Region"
+	D[pdTrace] << "Receiving Message from Federate: Unassociate Region"
 		   << endl ;
 	ddm->unassociateRegion(req->getObject(), req->getRegion(), e);
 	break ;
@@ -696,15 +708,16 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
 	break ;
 
       case Message::DDM_UNSUBSCRIBE_ATTRIBUTES:
-	D[pdTrace] << "Receiving Message from Federate: Unsubscribe Attributes"
-		   << endl ;
+	D[pdTrace] << "Receiving Message from Federate: Unsubscribe class "
+		   << req->getObjectClass() << endl ;
 	ddm->unsubscribeAttributes(req->getObjectClass(), req->getRegion(), e);
 	break ;
 
       case Message::DDM_SUBSCRIBE_INTERACTION:
 	D[pdTrace] << "Receiving Message from Federate: Subscribe Interaction"
 		   << endl ;
-	ddm->subscribe(req->getInteractionClass(), req->getRegion(), e);
+	ddm->subscribeInteraction(req->getInteractionClass(),
+				  req->getRegion(), e);
 	break ;
 
       case Message::DDM_UNSUBSCRIBE_INTERACTION:
@@ -739,7 +752,7 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
       default:
         D.Out(pdExcept,
               "Receiving Message from Federate, Unknown Type %d.", req->type);
-        throw RTIinternalError();
+        throw RTIinternalError("");
     }
 
     stat.federateService(req->type);
@@ -1088,4 +1101,4 @@ RTIA::processFederateRequest(Message *req)
 
 }} // namespace certi/rtia
 
-// $Id: RTIA_federate.cc,v 3.27 2003/11/13 10:52:20 breholee Exp $
+// $Id: RTIA_federate.cc,v 3.31 2005/04/30 16:38:39 breholee Exp $
