@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003, 2004  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
 // This file is part of CERTI
 //
@@ -18,7 +18,7 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: Federation.cc,v 3.37 2004/03/04 20:19:04 breholee Exp $
+// $Id: Federation.cc,v 3.45 2005/04/30 16:41:27 breholee Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -28,6 +28,7 @@
 #include "XmlParser.hh"
 #include "ObjectClassAttribute.hh"
 #include "PrettyDebug.hh"
+#include "LBTS.hh"
 
 #include <map>
 #include <fstream>
@@ -69,14 +70,14 @@ static PrettyDebug D("FEDERATION", __FILE__);
 #ifdef FEDERATION_USES_MULTICAST
 Federation::Federation(const char *federation_name,
                        FederationHandle federation_handle,
-                       SocketServer *socket_server,
-                       AuditFile *audit_server,
+                       SocketServer &socket_server,
+                       AuditFile &audit_server,
                        SocketMC *mc_link)
 #else
     Federation::Federation(const char *federation_name,
                            Handle federation_handle,
-                           SocketServer *socket_server,
-                           AuditFile *audit_server)
+                           SocketServer &socket_server,
+                           AuditFile &audit_server)
 #endif
     throw (CouldNotOpenRID, ErrorReadingRID, MemoryExhausted, SecurityError,
            RTIinternalError)
@@ -115,7 +116,6 @@ Federation::Federation(const char *federation_name,
 
     // Initialize the Security Server.
     server = new SecurityServer(socket_server, audit_server, handle);
-    if (server == NULL) throw RTIinternalError();
 
     // Read FOM File to initialize Root Object.
     root = new RootObject(server);
@@ -131,7 +131,7 @@ Federation::Federation(const char *federation_name,
 
         cout << "yes" << endl ;
 	int err = fedparser::build(filename.c_str(), root, verbose);
-	if (err) throw ErrorReadingFED();
+	if (err) throw ErrorReadingFED("");
 	    
         // Retrieve the FED file last modification time(for Audit)
         struct stat StatBuffer ;
@@ -140,11 +140,11 @@ Federation::Federation(const char *federation_name,
         if (stat(filename.c_str(), &StatBuffer) == 0) {
             MTimeBuffer = ctime(&StatBuffer.st_mtime);
             MTimeBuffer[strlen(MTimeBuffer) - 1] = 0 ; // Remove trailing \n
-            server->Audit->addToLinef("(Last modified %s)", MTimeBuffer);
+            server->audit << "(Last modified " << MTimeBuffer << ")" ;
         }
         else
-            server->Audit->addToLinef("(could not retrieve last modif time, "
-                                      "errno %d).", errno);
+            server->audit << "(could not retrieve last modif time, errno "
+			  << errno << ")." ;
     }
     else {
         cout << "no" << endl ;
@@ -159,7 +159,7 @@ Federation::Federation(const char *federation_name,
                 cout << "yes" << endl ;
 
                 XmlParser *parser = new XmlParser(root);
-                server->Audit->addToLinef(", XML File : %s", filename.c_str());
+                server->audit << ", XML File : " << filename.c_str() ;
 
                 try {
                     parser->parse(filename);
@@ -176,7 +176,7 @@ Federation::Federation(const char *federation_name,
             }
             else {
                 cout << "no" << endl ;
-		throw CouldNotOpenFED();
+		throw CouldNotOpenFED("");
 	    }
         }
     }
@@ -269,7 +269,7 @@ Federation::add(const char *federate_name, SocketTCP *tcp_link)
 
     try {
         this->getByName(federate_name);
-        throw FederateAlreadyExecutionMember();
+        throw FederateAlreadyExecutionMember("");
     }
     catch (FederateNotExecutionMember &e) {
         // Nothing to do.
@@ -285,13 +285,17 @@ Federation::add(const char *federate_name, SocketTCP *tcp_link)
     // its LBTS.
     NetworkMessage message ;
     try {
-        for (unsigned int i = 1 ; i <= regulators.size(); ++i) {
+	std::vector<LBTS::FederateClock> v ;
+	regulators.get(v);
+
+        for (unsigned int i = 0 ; i < v.size(); ++i) {
             message.type = NetworkMessage::MESSAGE_NULL ;
             message.federation = handle ;
+	    message.federate = v[i].first ;
+	    message.date = v[i].second ;
 
-            regulators.get(i, message.federate, message.date);
-
-            D.Out(pdTerm, "Sending NULL message(type %d) from %d to new federate.",
+            D.Out(pdTerm,
+		  "Sending NULL message(type %d) from %d to new federate.",
                   message.type, message.federate);
 
             message.write(tcp_link);
@@ -374,7 +378,7 @@ Federation::addRegulator(FederateHandle federate_handle, FederationTime time)
     msg.exception = e_NO_EXCEPTION ;
     msg.federation = handle ;
     msg.federate = federate_handle ;
-    msg.regulator = RTI_TRUE ;
+    msg.regulator = true ;
     msg.date = time ;
 
     this->broadcastAnyMessage(&msg, 0);
@@ -423,6 +427,7 @@ Federation::broadcastInteraction(FederateHandle federate_handle,
                                  ParameterValue *parameter_values,
                                  UShort list_size,
                                  FederationTime time,
+				 RegionHandle region_handle,
                                  const char *tag)
     throw (FederateNotExecutionMember,
            FederateNotPublishing,
@@ -435,12 +440,17 @@ Federation::broadcastInteraction(FederateHandle federate_handle,
     // It may throw FederateNotExecutionMember.
     this->check(federate_handle);
 
+    const RTIRegion *region = 0 ;
+    if (region_handle != 0)
+	region = root->getRegion(region_handle);
+
     root->Interactions->broadcastInteraction(federate_handle,
                                              interaction,
                                              parameter_handles,
                                              parameter_values,
                                              list_size,
                                              time,
+					     region,
                                              tag);
     D.Out(pdRequest, "Federation %d: Broadcasted Interaction %d from Federate "
           "%d nb params %d.", handle, interaction, federate_handle, list_size);
@@ -500,7 +510,7 @@ Federation::registerSynchronization(FederateHandle federate,
     i = synchronizationLabels.begin();
     for (; i != synchronizationLabels.end(); i++) {
         if (!strcmp((*i).first, label)) {
-            throw FederationAlreadyPaused(); // Label already pending.
+            throw FederationAlreadyPaused(""); // Label already pending.
         }
     }
 
@@ -792,7 +802,7 @@ Federation::empty() const
     if (list<Federate *>::empty())
         return true ;
     else
-        throw FederatesCurrentlyJoined();
+        throw FederatesCurrentlyJoined("");
 }
 
 // ----------------------------------------------------------------------------
@@ -809,7 +819,7 @@ Federation::check(FederateHandle federate_handle) const
             return true ;
     }
 
-    throw FederateNotExecutionMember();
+    throw FederateNotExecutionMember("");
 }
 
 // ----------------------------------------------------------------------------
@@ -957,7 +967,7 @@ Federation::remove(FederateHandle federate_handle)
 
     D.Out(pdExcept, "Federation %d could not remove unknown federate %d.",
           handle, federate_handle);
-    throw FederateNotExecutionMember();
+    throw FederateNotExecutionMember("");
 }
 
 // ----------------------------------------------------------------------------
@@ -1007,7 +1017,7 @@ Federation::removeRegulator(FederateHandle federate_handle)
     msg.exception = e_NO_EXCEPTION ;
     msg.federation = handle ;
     msg.federate = federate_handle ;
-    msg.regulator = RTI_FALSE ;
+    msg.regulator = false ;
     msg.date = 0 ;
 
     broadcastAnyMessage(&msg, 0);
@@ -1087,7 +1097,7 @@ Federation::subscribeInteraction(FederateHandle federate,
     this->check(federate);
 
     // It may throw *NotDefined
-    root->Interactions->subscribe(federate, interaction, sub);
+    root->Interactions->subscribe(federate, interaction, 0, sub);
     D.Out(pdRegister,
           "Federation %d: Federate %d(un)subscribes to Interaction %d.",
           handle, federate, interaction);
@@ -1100,8 +1110,7 @@ void
 Federation::subscribeObject(FederateHandle federate,
                             ObjectClassHandle object,
                             AttributeHandle *attributes,
-                            UShort list_size,
-                            bool sub)
+                            UShort list_size)
     throw (ObjectClassNotDefined,
            AttributeNotDefined,
            FederateNotExecutionMember,
@@ -1114,7 +1123,7 @@ Federation::subscribeObject(FederateHandle federate,
     this->check(federate);
 
     // It may throw *NotDefined
-    root->ObjectClasses->subscribe(federate, object, attributes, list_size, sub);
+    root->ObjectClasses->subscribe(federate, object, attributes, list_size);
     D.Out(pdRegister,
           "Federation %d: Federate %d(un)sub. to %d attrib. of ObjClass %d.",
           handle, federate, list_size, object);
@@ -1466,10 +1475,10 @@ Federation::deleteRegion(FederateHandle federate,
     this->check(federate);
 
     if (saveInProgress) {
-        throw SaveInProgress();
+        throw SaveInProgress("");
     }
     if (restoreInProgress) {
-        throw RestoreInProgress();
+        throw RestoreInProgress("");
     }
 
     // TODO: check RegionInUse
@@ -1488,7 +1497,7 @@ Federation::associateRegion(FederateHandle federate,
 {
     check(federate);
 
-    RegionImp *region = root->getRegion(the_handle);
+    RTIRegion *region = root->getRegion(the_handle);
 
     root->getObject(object)->unassociate(region);
 	
@@ -1507,7 +1516,7 @@ Federation::unassociateRegion(FederateHandle federate, ObjectHandle object,
 {
     check(federate);
 
-    RegionImp *region = root->getRegion(the_handle);
+    RTIRegion *region = root->getRegion(the_handle);
     root->getObject(object)->unassociate(region);
 }
 
@@ -1520,18 +1529,13 @@ Federation::subscribeAttributesWR(FederateHandle federate,
 				  unsigned short nb,
 				  AttributeHandle *attributes)
     throw (RegionNotKnown,
-	       SaveInProgress,
-	       RestoreInProgress,
-	       RTIinternalError)
+	   SaveInProgress,
+	   RestoreInProgress,
+	   RTIinternalError)
 {
     check(federate);
-
-    RegionImp *r = root->getRegion(region_handle);
-    
-    root->getObjectClass(c)->unsubscribe(federate, r);
-    for (int i = 0 ; i < nb ; ++i) {
-	root->getObjectClassAttribute(c, attributes[i])->subscribe(federate, r);
-    } 
+    root->ObjectClasses->subscribe(federate, c, attributes, nb,
+				   root->getRegion(region_handle));
 }
 
 // ----------------------------------------------------------------------------
@@ -1547,7 +1551,7 @@ Federation::unsubscribeAttributesWR(FederateHandle federate,
 {
     check(federate);
 
-    RegionImp *region = root->getRegion(region_handle);
+    RTIRegion *region = root->getRegion(region_handle);
 
     root->getObjectClass(object_class)->unsubscribe(federate, region);
 }
@@ -1564,7 +1568,7 @@ Federation::subscribeInteractionWR(FederateHandle federate,
 {
     check(federate);
 
-    RegionImp *region = root->getRegion(region_handle);
+    RTIRegion *region = root->getRegion(region_handle);
 
     root->getInteractionClass(interaction)->subscribe(federate, region);
 }
@@ -1581,11 +1585,54 @@ Federation::unsubscribeInteractionWR(FederateHandle federate,
 {
     check(federate);
 
-    RegionImp *region = root->getRegion(region_handle);
+    RTIRegion *region = root->getRegion(region_handle);
 
     root->getInteractionClass(interaction)->unsubscribe(federate, region);
 }
 
+// ----------------------------------------------------------------------------
+ObjectHandle
+Federation::registerObjectWithRegion(FederateHandle federate,
+				     ObjectClassHandle class_handle,
+				     const char *object_name,
+				     RegionHandle region_handle,
+				     int nb,
+				     AttributeHandle *attributes)
+    throw (ObjectClassNotDefined, ObjectClassNotPublished,
+	   AttributeNotDefined, AttributeNotPublished, RegionNotKnown,
+	   InvalidRegionContext, ObjectAlreadyRegistered,
+	   SaveInProgress, RestoreInProgress,
+	   RTIinternalError)
+{
+    check(federate);
+
+    // Register object
+    ObjectHandle object = objectHandles.provide();
+    D[pdDebug] << "Register object with region : Object " << object
+	       << ", class " << class_handle << ", region " << region_handle
+	       << std::endl ;
+    string strname = "" ;    // create a name if necessary
+    strname += object_name ? string(object_name) : "HLA" + object ;
+
+    root->registerObjectInstance(federate, class_handle, object, 
+				 strname.c_str());
+
+    D[pdDebug] << "- object \"" << strname.c_str()
+	       << "\" registered" << std::endl ;
+
+    // Associate region
+    RTIRegion *region = root->getRegion(region_handle);
+    root->getObject(object)->unassociate(region);
+	
+    for (int i = 0 ; i < nb ; ++i) {
+	root->getObjectAttribute(object, attributes[i])->associate(region);
+    }    
+
+    D[pdDebug] << "- " << nb << " attribute(s) associated with region "
+	       << region_handle << std::endl ;
+    return object ;
+}    
+
 // ----------------------------------------------------------------------------
 bool
 Federation::restoreXmlData()
@@ -1710,5 +1757,5 @@ Federation::saveXmlData()
 
 }} // namespace certi/rtig
 
-// $Id: Federation.cc,v 3.37 2004/03/04 20:19:04 breholee Exp $
+// $Id: Federation.cc,v 3.45 2005/04/30 16:41:27 breholee Exp $
 
