@@ -19,13 +19,13 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA
 //
-// $Id: AuditLine.cc,v 3.6 2003/06/27 17:26:28 breholee Exp $
+// $Id: AuditLine.cc,v 3.8 2004/05/18 13:18:53 breholee Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "AuditLine.hh"
 
-#include "RTItypes.hh"
+#include "certi.hh"
 
 #include <string>
 #include <time.h>
@@ -36,23 +36,26 @@ using std::string ;
 
 namespace certi {
 
-// ----------------------------------------------------------------------------
-//! addComment adds information to the comment parameter.
-void
-AuditLine::addComment(const char *str)
-{
-    comment += str ;
-}
-
 // ----------------------------------------------------------------------------
 //! AuditLine constructor.
 /*! Initialise internal parameters to null.
  */
 AuditLine::AuditLine()
-    : federation(0), federate(0), type(0), level(0), status(0)
+    : federation(0), federate(0), type(0), level(0), status(0),
+      modified(false), date(0)
 {
-    // Set the Date
-    date = time(NULL);
+}
+
+// ----------------------------------------------------------------------------
+//! AuditLine constructor.
+/*! 
+ */
+AuditLine::AuditLine(unsigned short event_type, unsigned short event_level,
+		     unsigned short event_status, string reason)
+    : federation(0), federate(0),
+      type(event_type), level(event_level), status(event_status),
+      modified(false), date(0), comment(reason)
+{    
 }
 
 // ----------------------------------------------------------------------------
@@ -61,6 +64,24 @@ AuditLine::~AuditLine()
 {
 }
 
+// ----------------------------------------------------------------------------
+//! addComment adds information to the comment parameter.
+void
+AuditLine::addComment(const std::string &str)
+{
+    comment += str ;
+    modified = true ;
+}
+
+// ----------------------------------------------------------------------------
+//! Finish the line with a status and reason
+void
+AuditLine::end(unsigned short event_status, const string reason)
+{
+    status = event_status ;    
+    addComment(reason);
+}
+
 // ----------------------------------------------------------------------------
 //! Write a line to the audit file
 /*! Write module writes a line to the AuditFile stream. Line ends by a newline.
@@ -83,7 +104,28 @@ AuditLine::write(ofstream &audit_file)
     audit_file.flush();
 }
 
+void 
+AuditLine::setFederation(Handle h)
+{ 
+    federation = h ; 
+    modified = true ;
+}
+
+void 
+AuditLine::setFederate(FederateHandle h) 
+{ 
+    federate = h ; 
+    modified = true ;
+}
+
+void 
+AuditLine::setLevel(unsigned short l) 
+{
+    level = l ; 
+    modified = true ; 
+}
+
 } // namespace certi
 
-// $Id: AuditLine.cc,v 3.6 2003/06/27 17:26:28 breholee Exp $
+// $Id: AuditLine.cc,v 3.8 2004/05/18 13:18:53 breholee Exp $
 
