@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003, 2004  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
 // This file is part of CERTI-libCERTI
 //
@@ -19,7 +19,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA
 //
-// $Id: Interaction.cc,v 3.19 2004/03/04 20:19:05 breholee Exp $
+// $Id: Interaction.cc,v 3.26 2005/04/30 16:52:44 breholee Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -66,20 +66,20 @@ Interaction::addParametersToChild(Interaction *the_child)
     // The Parameter List is read backward to respect the same attribute order
     // for the child (Parameters are inserted at the beginning of the list)
     Parameter *child = NULL ;
-    list<Parameter *>::reverse_iterator p ;
-    for (p = parameterSet.rbegin(); p != parameterSet.rend(); p++) {
-        assert((*p) != NULL);
+    list<Parameter *>::reverse_iterator it ;
+    for (it = parameterSet.rbegin(); it != parameterSet.rend(); it++) {
+        assert((*it) != NULL);
 
-        child = new Parameter(*p);
+        child = new Parameter(**it);
         assert(child != NULL);
 
         D.Out(pdProtocol,
               "ObjectClass %u adding new attribute %d to child class %u.",
-              handle, (*p)->getHandle(), the_child->handle);
+              handle, (*it)->getHandle(), the_child->handle);
 
-        the_child->addParameter(child, RTI_TRUE);
+        the_child->addParameter(child, true);
 
-        if (child->getHandle() != (*p)->getHandle())
+        if (child->getHandle() != (*it)->getHandle())
             throw RTIinternalError("Error while copying child's attributes.");
     }
 }
@@ -91,7 +91,8 @@ Interaction::addParametersToChild(Interaction *the_child)
   See InteractionSet::SendInteraction.
 */
 void
-Interaction::broadcastInteractionMessage(InteractionBroadcastList *ibList)
+Interaction::broadcastInteractionMessage(InteractionBroadcastList *ibList,
+					 const RTIRegion *region)
 {
     // 1. Set InteractionHandle to local class Handle.
     ibList->message->interactionClass = handle ;
@@ -109,12 +110,7 @@ Interaction::broadcastInteractionMessage(InteractionBroadcastList *ibList)
     }
 
     // 3. Add Interaction subscribers to the list.
-    list<Subscriber *>::iterator s ;
-    for (s = subscribers.begin(); s != subscribers.end(); s++) {
-        D.Out(pdDebug, "Adding federate %d to BroadcastList.",
-              (*s)->getHandle());
-        ibList->addFederate((*s)->getHandle());
-    }
+    addFederatesIfOverlap(*ibList, region);
 
     // 4. Send pending messages.
     D.Out(pdDebug, "Calling SendPendingMessage...");
@@ -134,7 +130,7 @@ Interaction::changeTransportationType(TransportType new_type,
         throw FederateNotPublishing("Change Interaction Transport Type.");
 
     if ((new_type != RELIABLE) && (new_type != BEST_EFFORT))
-        throw InvalidTransportType();
+        throw InvalidTransportType("");
 
     transport = new_type ;
 
@@ -154,7 +150,7 @@ Interaction::changeOrderType(OrderType new_order, FederateHandle the_handle)
         throw FederateNotPublishing("Change Interaction Order Type.");
 
     if ((new_order != RECEIVE) && (new_order != TIMESTAMP))
-        throw InvalidOrderType();
+        throw InvalidOrderType("");
 
     D.Out(pdInit, "Interaction %d: New Order type is %d.", handle, order);
 }
@@ -199,27 +195,8 @@ Interaction::~Interaction()
         parameterSet.pop_front();
     }
 
-    // Deleting Publishers
     if (!publishers.empty())
-        D.Out(pdError,
-              "Interaction %d: publishers list not empty at termination.",
-              handle);
-
-    while (!publishers.empty()) {
-        delete publishers.front();
-        publishers.pop_front();
-    }
-
-    // Deleting Subscribers
-    if (!subscribers.empty())
-        D.Out(pdError,
-              "Interaction %d: Subscribers list not empty at termination.",
-              handle);
-
-    while (!subscribers.empty()) {
-        delete subscribers.front();
-        subscribers.pop_front();
-    }
+        D.Out(pdError, "Interaction %d: publishers list not empty at termination.", handle);
 
     // Deleting Sons
     while (!children.empty()) {
@@ -228,33 +205,13 @@ Interaction::~Interaction()
 }
 
 // ----------------------------------------------------------------------------
-//! Delete a publisher with rank (private module).
+//! Delete a publisher with rank
 void
 Interaction::deletePublisher(FederateHandle fed)
 {
-    list<Publisher *>::iterator p ;
-    for (p = publishers.begin(); p != publishers.end(); ++p) {
-        if ((*p)->getHandle() == fed) {
-            delete (*p);
-            publishers.erase(p);
-            return ;
-        }
-    }
-}
-
-// ----------------------------------------------------------------------------
-//! Delete a subscriber with rank (private module).
-void
-Interaction::deleteSubscriber(FederateHandle fed, RegionImp *region)
-{
-    list<Subscriber *>::iterator s ;
-    for (s = subscribers.begin(); s != subscribers.end(); ++s) {
-        if ((*s)->match(fed, region)) {
-            delete *s ;
-            subscribers.erase(s);
-            return ;
-        }
-    }
+    PublishersList::iterator it = publishers.find(fed);
+    if (it != publishers.end())
+	publishers.erase(it);
 }
 
 // ----------------------------------------------------------------------------
@@ -285,7 +242,7 @@ Interaction::display() const
 }
 
 // ----------------------------------------------------------------------------
-//! Returns the parameter by its handle (private module).
+//! Returns the parameter by its handle
 Parameter*
 Interaction::getParameterByHandle(ParameterHandle the_handle) const
     throw (InteractionParameterNotDefined, RTIinternalError)
@@ -296,7 +253,7 @@ Interaction::getParameterByHandle(ParameterHandle the_handle) const
             return (*p);
     }
 
-    throw InteractionParameterNotDefined();
+    throw InteractionParameterNotDefined("");
 }
 
 // ----------------------------------------------------------------------------
@@ -307,11 +264,11 @@ Interaction::getParameterHandle(const char *the_name) const
 {
     list<Parameter *>::const_iterator p ;
     for (p = parameterSet.begin(); p != parameterSet.end(); p++) {
-        if (strcmp((*p)->getName(), the_name) == 0)
+        if ((*p)->getName() == the_name)
             return (*p)->getHandle();
     }
 
-    throw NameNotFound();
+    throw NameNotFound("");
 }
 
 // ----------------------------------------------------------------------------
@@ -321,29 +278,7 @@ Interaction::getParameterName(ParameterHandle the_handle) const
     throw (InteractionParameterNotDefined,
            RTIinternalError)
 {
-    return getParameterByHandle(the_handle)->getName();
-}
-
-// ----------------------------------------------------------------------------
-//! Return true if federate has subscribed to this attribute w/ region
-bool
-Interaction::isSubscribed(FederateHandle fed, RegionImp *region)
-{
-    list<Subscriber *>::iterator s ;
-    for (s = subscribers.begin(); s != subscribers.end(); ++s) {
-        if ((*s)->match(fed, region)) {
-	    return true ;
-        }
-    }
-    return false ;
-}
-
-// ----------------------------------------------------------------------------
-//! Return true if federate has subscribed to this attribute w/ default region
-bool
-Interaction::isSubscribed(FederateHandle fed)
-{
-    return isSubscribed(fed, 0);
+    return getParameterByHandle(the_handle)->getName().c_str();
 }
 
 // ----------------------------------------------------------------------------
@@ -351,13 +286,7 @@ Interaction::isSubscribed(FederateHandle fed)
 bool
 Interaction::isPublishing(FederateHandle fed)
 {
-    list<Publisher *>::iterator p ;
-    for (p = publishers.begin(); p != publishers.end(); ++p) {
-        if ((*p)->getHandle() == fed) {
-	    return true ;
-        }
-    }
-    return false ;
+    return publishers.find(fed) != publishers.end();
 }
 
 // ----------------------------------------------------------------------------
@@ -374,7 +303,7 @@ Interaction::isReady(FederateHandle federate_handle,
 {
     // Is Federate Publishing Interaction?
     if (!isPublishing(federate_handle))
-        throw FederateNotPublishing();
+        throw FederateNotPublishing("");
 
     // Are Parameters Defined?
     for (UShort i = 0 ; i < list_size ; i++)
@@ -408,15 +337,11 @@ Interaction::publish(FederateHandle the_handle)
     checkFederateAccess(the_handle, (char *) "Publish");
     
     if (!isPublishing(the_handle)) {
-	D.Out(pdInit,
-	      "Interaction %d: Added Federate %d to publishers list.",
-	      handle, the_handle);
-	publishers.push_front(new Publisher(the_handle));
+	D.Out(pdInit, "Interaction %d: Added Federate %d to publishers list.", handle, the_handle);
+	publishers.insert(the_handle);
     }
     else
-	D.Out(pdError,
-	      "Interaction %d: Inconsistent publish request from"
-	      " Federate %d.", handle, the_handle);
+	D.Out(pdError, "Interaction %d: Inconsistent publish request from Federate %d.", handle, the_handle);
 }
 
 // ----------------------------------------------------------------------------
@@ -432,7 +357,7 @@ Interaction::unpublish(FederateHandle the_handle)
 	deletePublisher(the_handle);
     }
     else {
-	throw FederateNotPublishing();
+	throw FederateNotPublishing("");
     }
 }
 
@@ -446,6 +371,7 @@ Interaction::sendInteraction(FederateHandle federate_handle,
                              ParameterValue *value_list,
                              UShort list_size,
                              FederationTime time,
+			     const RTIRegion *region,
                              const char *the_tag)
     throw (FederateNotPublishing,
            InteractionClassNotDefined,
@@ -454,7 +380,7 @@ Interaction::sendInteraction(FederateHandle federate_handle,
 {
     // Pre-conditions checking
     if (!isPublishing(federate_handle))
-        throw FederateNotPublishing();
+        throw FederateNotPublishing("");
 
     // Prepare and Broadcast message for this class
     InteractionBroadcastList *ibList = NULL ;
@@ -478,7 +404,7 @@ Interaction::sendInteraction(FederateHandle federate_handle,
         D.Out(pdProtocol, "Preparing broadcast list.");
         ibList = new InteractionBroadcastList(answer);
 
-        broadcastInteractionMessage(ibList);
+        broadcastInteractionMessage(ibList, region);
     }
     else
         // SendInteraction should not be called on the RTIA.
@@ -515,66 +441,12 @@ Interaction::setName(const char *new_name)
 void
 Interaction::setLevelId(SecurityLevelID new_levelID)
 {
-    if (server->dominates(new_levelID, id) == RTI_FALSE)
+    if (!server->dominates(new_levelID, id))
         throw SecurityError("Attempt to lower interaction class level.");
 
     id = new_levelID ;
 }
 
-// ----------------------------------------------------------------------------
-//! subscribe
-void
-Interaction::subscribe(FederateHandle fed, RegionImp *region)
-    throw (FederateNotSubscribing, RTIinternalError, SecurityError)
-{
-    checkFederateAccess(fed, "Subscribe");
-
-    if (!isSubscribed(fed, region)) {
-	subscribers.push_front(new Subscriber(fed, region));
-	D.Out(pdInit,
-	      "Parameter %d: Added Federate %d to subscribers list.",
-	      handle, fed);
-    }
-    else
-	D.Out(pdError,
-	      "Parameter %d: Unconsistent subscribe request from "
-	      "federate %d.", handle, fed);    
-}
-
-// ----------------------------------------------------------------------------
-//! subscribe
-void
-Interaction::subscribe(FederateHandle fed)
-    throw (FederateNotSubscribing, RTIinternalError, SecurityError)
-{
-    subscribe(fed, 0);
-}
-
-// ----------------------------------------------------------------------------
-//! unsubscribe
-void
-Interaction::unsubscribe(FederateHandle fed, RegionImp *region)
-    throw (FederateNotSubscribing, RTIinternalError, SecurityError)
-{
-    if (isSubscribed(fed)) {
-	deleteSubscriber(fed, region);
-	D.Out(pdTerm,
-	      "Parameter %d: Removed Federate %d from subscribers list.",
-	      handle, fed);
-    }
-    else
-	throw FederateNotSubscribing();
-}
-
-// ----------------------------------------------------------------------------
-//! unsubscribe
-void
-Interaction::unsubscribe(FederateHandle fed)
-    throw (FederateNotSubscribing, RTIinternalError, SecurityError)
-{
-    unsubscribe(fed, 0);
-}
-
 // ----------------------------------------------------------------------------
 // setSpace
 void
@@ -593,4 +465,4 @@ Interaction::getSpace()
 
 } // namespace certi
 
-// $Id: Interaction.cc,v 3.19 2004/03/04 20:19:05 breholee Exp $
+// $Id: Interaction.cc,v 3.26 2005/04/30 16:52:44 breholee Exp $
