@@ -1,31 +1,26 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
-// This file is part of CERTI-libCERTI
-//
-// CERTI-libCERTI is free software ; you can redistribute it and/or
+// This program is free software ; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public License
 // as published by the Free Software Foundation ; either version 2 of
 // the License, or (at your option) any later version.
 //
-// CERTI-libCERTI is distributed in the hope that it will be useful, but
+// This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY ; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 // Lesser General Public License for more details.
 //
 // You should have received a copy of the GNU Lesser General Public
 // License along with this program ; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA
-//
-// $Id: Message.cc,v 3.23 2003/11/10 14:43:01 breholee Exp $
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "Message.hh"
 
-#include "RTItypes.hh"
+#include "certi.hh"
 #include "PrettyDebug.hh"
 #include "GAV.hh"
 #include "converter.hh"
@@ -41,7 +36,7 @@ namespace certi {
 
 Message::Message()
 {
-    date = 0.0 ;
+    fed_time.setZero();
     exception = e_NO_EXCEPTION ;
     exceptionReason[0] = '\0' ;
     federateName[0] = '\0' ;
@@ -53,15 +48,6 @@ Message::Message()
 }
 
 // ----------------------------------------------------------------------------
-//! Destructor
-//
-Message::~Message()
-{
-}
-
-// ----------------------------------------------------------------------------
-//! getValue
-//
 char *Message::getValue(int Rank, char *Value) const
     throw (RTIinternalError)
 {
@@ -81,8 +67,6 @@ char *Message::getValue(int Rank, char *Value) const
 }
 
 // ----------------------------------------------------------------------------
-// getValueArray
-//
 ParameterValue *
 Message::getValueArray()
 {
@@ -102,9 +86,8 @@ Message::getValueArray()
 }
 
 // ----------------------------------------------------------------------------
-// setLabel
-//
-void Message::setLabel(const char *NewLabel)
+void
+Message::setLabel(const char *NewLabel)
 {
     if (strlen(NewLabel) > MAX_USER_TAG_LENGTH)
         throw ValueLengthExceeded("Label too long to fit in Message.");
@@ -113,8 +96,6 @@ void Message::setLabel(const char *NewLabel)
 }
 
 // ----------------------------------------------------------------------------
-// setName
-//
 void
 Message::setName(const char *NewName)
 {
@@ -125,15 +106,12 @@ Message::setName(const char *NewName)
 }
 
 // ----------------------------------------------------------------------------
-// setType
-//
 void
 Message::setType(Type t)
 {
     type = t ;
 }
 // ----------------------------------------------------------------------------
-// setNbExtents
 void
 Message::setNumber(unsigned long nb)
 {
@@ -141,7 +119,6 @@ Message::setNumber(unsigned long nb)
 }
 
 // ----------------------------------------------------------------------------
-// setSpace
 void
 Message::setSpace(SpaceHandle handle)
 {
@@ -149,7 +126,6 @@ Message::setSpace(SpaceHandle handle)
 }
 
 // ----------------------------------------------------------------------------
-// setRegion
 void
 Message::setRegion(long handle)
 {
@@ -157,7 +133,6 @@ Message::setRegion(long handle)
 }
 
 // ----------------------------------------------------------------------------
-// setDimension
 void
 Message::setDimension(DimensionHandle handle)
 {
@@ -165,7 +140,6 @@ Message::setDimension(DimensionHandle handle)
 }
 
 // ----------------------------------------------------------------------------
-// setInteractionClass
 void
 Message::setInteractionClass(InteractionClassHandle handle)
 {
@@ -173,7 +147,6 @@ Message::setInteractionClass(InteractionClassHandle handle)
 }
 
 // ----------------------------------------------------------------------------
-// setObjectClass
 void
 Message::setObjectClass(ObjectClassHandle handle)
 {
@@ -181,7 +154,6 @@ Message::setObjectClass(ObjectClassHandle handle)
 }
 
 // ----------------------------------------------------------------------------
-// setAttribute
 void
 Message::setAttribute(AttributeHandle handle)
 {
@@ -190,7 +162,7 @@ Message::setAttribute(AttributeHandle handle)
 
 // ----------------------------------------------------------------------------
 void
-Message::setResignAction(ResignAction the_action)
+Message::setResignAction(RTI::ResignAction the_action)
 {
     resignAction = the_action ;
 }
@@ -204,14 +176,14 @@ Message::setFederationTimeDelta(FederationTimeDelta the_lookahead)
 
 // ----------------------------------------------------------------------------
 void
-Message::setFedTime(const FedTime &the_time)
+Message::setFedTime(const RTI::FedTime &the_time)
 {
-    date = (FederationTime) (RTIfedTime(the_time)._fedTime);
+    fed_time = dynamic_cast<const RTIfedTime &>(the_time);
 }
 
 // ----------------------------------------------------------------------------
 void
-Message::setLookahead(const FedTime& the_lookahead)
+Message::setLookahead(const RTI::FedTime& the_lookahead)
 {
     lookahead = (FederationTimeDelta) ((RTIfedTime&) the_lookahead).getTime();
 }
@@ -220,12 +192,12 @@ Message::setLookahead(const FedTime& the_lookahead)
 void
 Message::setFederationTime(FederationTime the_time)
 {
-    date = the_time ;
+    fed_time = RTIfedTime(the_time);
 }
 
 // ----------------------------------------------------------------------------
 void
-Message::setBoolean(Boolean the_bool)
+Message::setBoolean(bool the_bool)
 {
     boolean = the_bool ;
 }
@@ -239,14 +211,14 @@ Message::setObject(ObjectHandle the_object)
 
 // ----------------------------------------------------------------------------
 void
-Message::setTransportation(TransportationHandle the_transport)
+Message::setTransportation(RTI::TransportationHandle the_transport)
 {
     transport = ((the_transport == 1) ? RELIABLE : BEST_EFFORT);
 }
 
 // ----------------------------------------------------------------------------
 void
-Message::setOrdering(OrderingHandle the_ordering)
+Message::setOrdering(RTI::OrderingHandle the_ordering)
 {
     order = ((the_ordering == 1) ? RECEIVE : TIMESTAMP);
 }
@@ -290,7 +262,7 @@ AttributeHandleSet*
 Message::getAHS() const
 {
     AttributeHandleSet *attributeSet ;
-    attributeSet = AttributeHandleSetFactory::create(handleArraySize);
+    attributeSet = RTI::AttributeHandleSetFactory::create(handleArraySize);
 
     for (int i = 0 ; i < handleArraySize ; i++) {
         attributeSet->add(handleArray[i]);
@@ -322,22 +294,10 @@ Message::setAHS(const AttributeHandle *attr, int size)
 }
 
 // ----------------------------------------------------------------------------
-void
-Message::setRegions(const RegionImp **reg, int size)
-{
-    regions.resize(size);
-    
-    for (int i = 0 ; i < size ; ++i) {
-        regions[i] = reg[i]->getHandle();
-    }
-}
-
-// ----------------------------------------------------------------------------
-AttributeHandleValuePairSet*
+RTI::AttributeHandleValuePairSet *
 Message::getAHVPS() const
 {
-    CAttributeHandleValuePairSet * theAttributes ;
-    theAttributes = new CAttributeHandleValuePairSet ;
+    CAttributeHandleValuePairSet theAttributes ;
 
     for (int i = 0 ; i < handleArraySize ; i++) {
         CAttributeHandleValuePair *att = new CAttributeHandleValuePair ;
@@ -352,25 +312,24 @@ Message::getAHVPS() const
         // BUG: Federate is expecting to find value type.
         strcpy(att->_value.type, "");
 
-        theAttributes->add(att);
+        theAttributes.add(att);
     }
 
-    return theAttributes->toAHVPS();
+    return theAttributes.toAHVPS();
 }
 
 // ----------------------------------------------------------------------------
 void
-Message::setAHVPS(const AttributeHandleValuePairSet &the_attributes)
+Message::setAHVPS(const RTI::AttributeHandleValuePairSet &the_attributes)
 {
     ULong length ;
-    CAttributeHandleValuePairSet *theAttributes_aux ;
-    theAttributes_aux = new CAttributeHandleValuePairSet(the_attributes);
-    CAttributeHandleValuePair *tmp ;
+    CAttributeHandleValuePairSet theAttributes_aux(the_attributes);
 
-    handleArraySize = theAttributes_aux->_size ;
+    const int size(theAttributes_aux._size);
+    handleArraySize = size ;
 
-    for (int i = 0 ; i < theAttributes_aux->_size ; i++) {
-        tmp = theAttributes_aux->getIeme(i);
+    for (int i = 0 ; i < size ; i++) {
+        CAttributeHandleValuePair *tmp = theAttributes_aux.getIeme(i);
         handleArray[i] = tmp->_attrib ;
 
         // codage
@@ -380,16 +339,15 @@ Message::setAHVPS(const AttributeHandleValuePairSet &the_attributes)
         char *value = new char[length] ;
         objectToString(tmp->_value.value, tmp->_value.length, value);
         setValue(i, value);
+        delete[] value;
     }
-    delete theAttributes_aux ;
 }
 
 // ----------------------------------------------------------------------------
-ParameterHandleValuePairSet*
+RTI::ParameterHandleValuePairSet *
 Message::getPHVPS() const
 {
-    CParameterHandleValuePairSet* theParameters ;
-    theParameters = new CParameterHandleValuePairSet ;
+    CParameterHandleValuePairSet theParameters ;
 
     for (int i = 0 ; i < handleArraySize ; i++) {
         CParameterHandleValuePair *par = new CParameterHandleValuePair ;
@@ -405,25 +363,24 @@ Message::getPHVPS() const
         // BUG: Federate is expecting to find value type.
         strcpy(par->_value.type, "");
 
-        theParameters->add(par);
+        theParameters.add(par);
     }
 
-    return theParameters->toPHVPS();
+    return theParameters.toPHVPS();
 }
 
 // ----------------------------------------------------------------------------
 void
-Message::setPHVPS(const ParameterHandleValuePairSet &the_parameters)
+Message::setPHVPS(const RTI::ParameterHandleValuePairSet &the_parameters)
 {
     ULong length ;
-    CParameterHandleValuePairSet *theParameters_aux ;
-    theParameters_aux = new CParameterHandleValuePairSet(the_parameters);
-    CParameterHandleValuePair *tmp ;
+    CParameterHandleValuePairSet theParameters_aux(the_parameters);
 
-    handleArraySize = theParameters_aux->_size ;
+    const int size(theParameters_aux._size);
+    handleArraySize = size ;
 
-    for (int i = 0 ; i < theParameters_aux->_size ; i++) {
-        tmp = theParameters_aux->getIeme(i);
+    for (int i = 0 ; i < size ; i++) {
+        CParameterHandleValuePair *tmp = theParameters_aux.getIeme(i);
         handleArray[i] = tmp->_param ;
 
         // codage
@@ -433,9 +390,8 @@ Message::setPHVPS(const ParameterHandleValuePairSet &the_parameters)
         char *value = new char[length] ;
         objectToString(tmp->_value.value, tmp->_value.length, value);
         setValue(i, value);
+        delete[] value;
     }
-
-    delete theParameters_aux ;
 }
 
 // ----------------------------------------------------------------------------
@@ -544,7 +500,7 @@ Message &
 Message::operator=(const Message& msg)
 {
     type = msg.type ;
-    date = msg.date ;
+    fed_time = msg.fed_time ;
     boolean = msg.boolean ;
     lookahead = msg.lookahead ;
     exception = msg.exception ;
@@ -575,7 +531,6 @@ Message::operator=(const Message& msg)
     transport = msg.transport ;
     order = msg.order ;
 
-    eventRetraction.theEventTime = msg.eventRetraction.theEventTime ;
     eventRetraction.theSerialNumber = msg.eventRetraction.theSerialNumber ;
     eventRetraction.sendingFederate = msg.eventRetraction.sendingFederate ;
 
@@ -601,7 +556,7 @@ Message::display(char *s)
 {
     printf(" --- MESSAGE --- %s ---\n", s);
     printf(" type=%d:\n", type);
-    printf(" date=%f:\n", date);
+    printf(" date=%f:\n", fed_time.getTime());
     printf(" exception=%d:\n", exception);
     printf(" objectClass=%ld:\n", objectClass);
     printf(" interactionClass=%ld:\n", interactionClass);
@@ -616,5 +571,3 @@ Message::display(char *s)
 }
 
 } // namespace certi
-
-// $Id: Message.cc,v 3.23 2003/11/10 14:43:01 breholee Exp $
