@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
 // This file is part of CERTI-libCERTI
 //
@@ -19,7 +19,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA
 //
-// $Id: ObjectClassSet.cc,v 3.14 2003/05/23 13:21:48 breholee Exp $
+// $Id: ObjectClassSet.cc,v 3.20 2005/04/30 17:16:08 breholee Exp $
 // ----------------------------------------------------------------------------
 
 #include "ObjectClassSet.hh"
@@ -35,10 +35,11 @@
 using std::list ;
 using std::cout ;
 using std::endl ;
+using std::string ;
 
 namespace certi {
 
-static pdCDebug D("OBJECTCLASSSET", "(ObjClSet) - ");
+static pdCDebug D("OBJECTCLASSSET", __FILE__);
 
 // ----------------------------------------------------------------------------
 //! The class is not allocated, only the pointer is memorized.
@@ -53,24 +54,16 @@ ObjectClassSet::addClass(ObjectClass *newClass)
 }
 
 // ----------------------------------------------------------------------------
-/*! Build a Parent-Child relation between two object class, by setting the
-  Child's Parent handle, and registering the child in the Parent's sonSet.
-  Also copy all parent's Attributes in the child Class.
-*/
+/** Build an inheritance relation between two object classes.
+ */
 void
-ObjectClassSet::buildParentRelation(ObjectClass *child, ObjectClass *parent)
+ObjectClassSet::buildParentRelation(ObjectClass *subclass,
+				    ObjectClass *superclass)
 {
-    // Register Parent to Son
-    child->Father = parent->getHandle();
-
-    // Transfer Security Level
-    child->setLevelId(parent->getLevelId());
-
-    // Register Son to Parent
-    parent->sonSet.push_front(child->getHandle());
-
-    // Copy Parent Attribute into Child class.
-    parent->addAttributesToChild(child);
+    subclass->setSuperclass(superclass->getHandle());
+    subclass->setLevelId(superclass->getLevelId());
+    superclass->addSubclass(subclass);
+    superclass->addAttributesToChild(subclass);
 }
 
 // ----------------------------------------------------------------------------
@@ -116,7 +109,7 @@ ObjectClassSet::deleteObject(FederateHandle federate,
     ObjectClassHandle current_class = 0 ;
     if (ocbList != 0) {
 
-        current_class = oclass->Father ;
+        current_class = oclass->getSuperclass();
 
         while (current_class) {
             D.Out(pdRegister,
@@ -127,7 +120,7 @@ ObjectClassSet::deleteObject(FederateHandle federate,
             oclass = getWithHandle(current_class);
             oclass->broadcastClassMessage(ocbList);
 
-            current_class = oclass->Father ;
+            current_class = oclass->getSuperclass();
         }
 
         delete ocbList ;
@@ -159,7 +152,7 @@ ObjectClassSet::getAttributeHandle(const char *the_name,
     ObjectClass *objectClass = 0 ;
 
     if (the_name == 0)
-        throw RTIinternalError();
+        throw RTIinternalError("");
 
     D.Out(pdRequest, "Looking for attribute \"%s\" of class %u...",
           the_name, the_class);
@@ -196,7 +189,6 @@ ObjectClass *
 ObjectClassSet::getInstanceClass(ObjectHandle theObjectHandle) const
     throw (ObjectNotKnown)
 {
-    // Try to find Instance's Class
     list<ObjectClass *>::const_iterator i ;
     for (i = begin(); i != end(); i++) {
         if ((*i)->isInstanceInClass(theObjectHandle) == true)
@@ -205,32 +197,45 @@ ObjectClassSet::getInstanceClass(ObjectHandle theObjectHandle) const
 
     D.Out(pdExcept, "Object instance %d not found in any object class.",
           theObjectHandle);
-    throw ObjectNotKnown();
+    throw ObjectNotKnown("");
+}
+
+// ----------------------------------------------------------------------------
+/** Get object
+ */
+Object *
+ObjectClassSet::getObject(ObjectHandle h) const
+    throw (ObjectNotKnown)
+{
+    list<ObjectClass *>::const_iterator i ;
+    for (i = begin(); i != end(); i++) {
+	try {
+	    Object *object = (*i)->getInstanceWithID(h);
+	    return object ;
+	}
+	catch (ObjectNotKnown &e) {
+	}
+    }
+    throw ObjectNotKnown("");
 }
 
 // ----------------------------------------------------------------------------
 //! getObjectClassHandle.
 ObjectClassHandle
-ObjectClassSet::getObjectClassHandle(const char *the_name) const
-    throw (NameNotFound, RTIinternalError)
+ObjectClassSet::getObjectClassHandle(string class_name) const
+    throw (NameNotFound)
 {
-    if (the_name == 0)
-        throw RTIinternalError();
-
-    D.Out(pdRequest, "Looking for class \"%s\"...", the_name);
-
     list<ObjectClass *>::const_iterator i ;
     for (i = begin(); i != end(); i++) {
-        if (strcmp((*i)->getName(), the_name) == 0)
+        if ((*i)->getName() == class_name)
             return (*i)->getHandle();
     }
-
-    throw NameNotFound();
+    throw NameNotFound("");
 }
 
 // ----------------------------------------------------------------------------
 //! getObjectClassName.
-const char *
+string
 ObjectClassSet::getObjectClassName(ObjectClassHandle the_handle) const
     throw (ObjectClassNotDefined,
            RTIinternalError)
@@ -276,7 +281,7 @@ void ObjectClassSet::killFederate(FederateHandle theFederate)
 
             // Broadcast RemoveObject message recursively
             if (ocbList != 0) {
-                currentClass = (*i)->Father ;
+                currentClass = (*i)->getSuperclass();
                 D.Out(pdExcept, "List not NULL");
                 while (currentClass != 0) {
                     D.Out(pdRegister,
@@ -288,7 +293,7 @@ void ObjectClassSet::killFederate(FederateHandle theFederate)
 
                     (*i)->broadcastClassMessage(ocbList);
 
-                    currentClass = (*i)->Father ;
+                    currentClass = (*i)->getSuperclass();
                 }
 
                 delete ocbList ;
@@ -314,7 +319,7 @@ ObjectClassSet::publish(FederateHandle theFederateHandle,
     // It may throw ObjectClassNotDefined
     ObjectClass *theClass = getWithHandle(theClassHandle);
 
-    if (PubOrUnpub == RTI_TRUE)
+    if (PubOrUnpub)
         D.Out(pdInit, "Federate %d attempts to publish Object Class %d.",
               theFederateHandle, theClassHandle);
     else
@@ -328,30 +333,6 @@ ObjectClassSet::publish(FederateHandle theFederateHandle,
                       PubOrUnpub);
 }
 
-// ----------------------------------------------------------------------------
-//! recursiveDiscovering.
-void
-ObjectClassSet::recursiveDiscovering(ObjectClassHandle theClassHandle,
-                                     FederateHandle theFederate,
-                                     ObjectClassHandle theOriginalClass)
-{
-    // It may throw ObjectClassNotDefined
-    ObjectClass *theClass = getWithHandle(theClassHandle);
-
-    D.Out(pdInit, "Recursive Discovering on class %d for Federate %d.",
-          theClassHandle, theFederate);
-
-    Boolean result ;
-    result = theClass->sendDiscoverMessages(theFederate, theOriginalClass);
-
-    if (result == RTI_TRUE) {
-        list<ObjectClassHandle>::const_iterator i = theClass->sonSet.begin();
-        for (; i != theClass->sonSet.end(); i++) {
-            recursiveDiscovering((*i), theFederate, theOriginalClass);
-        }
-    }
-}
-
 // ----------------------------------------------------------------------------
 //! registerInstance.
 void
@@ -377,7 +358,7 @@ ObjectClassSet::registerObjectInstance(FederateHandle the_federate,
     // Broadcast DiscoverObject message recursively
     if (ocbList != 0) {
 
-        currentClass = theClass->Father ;
+        currentClass = theClass->getSuperclass();
 
         while (currentClass != 0) {
             D.Out(pdRegister,
@@ -389,7 +370,7 @@ ObjectClassSet::registerObjectInstance(FederateHandle the_federate,
 
             theClass->broadcastClassMessage(ocbList);
 
-            currentClass = theClass->Father ;
+            currentClass = theClass->getSuperclass();
         }
 
         delete ocbList ;
@@ -400,94 +381,72 @@ ObjectClassSet::registerObjectInstance(FederateHandle the_federate,
 }
 
 // ----------------------------------------------------------------------------
-//! subscribe.
+/** Subscribes a federate to a set of attributes with a region. Sends the
+    discovery messages if necessary.
+    @param federate Federate to subscribe
+    @param class_handle Class to be subscribed
+    @param attributes List of attributes to be subscribed
+    @param nb Number of attributes
+    @param region Subscription region (0 for default)
+ */
 void
-ObjectClassSet::subscribe(FederateHandle theFederateHandle,
-                          ObjectClassHandle theClassHandle,
-                          AttributeHandle *theAttributeList,
-                          UShort theListSize,
-                          bool SubOrUnsub)
-    throw (ObjectClassNotDefined,
-           AttributeNotDefined,
-           RTIinternalError,
+ObjectClassSet::subscribe(FederateHandle federate,
+                          ObjectClassHandle class_handle,
+                          AttributeHandle *attributes,
+                          int nb,
+			  const RTIRegion *region)
+    throw (ObjectClassNotDefined, AttributeNotDefined, RTIinternalError,
            SecurityError)
 {
-    // It may throw ObjectClassNotDefined
-    ObjectClass *theClass = getWithHandle(theClassHandle);
+    ObjectClass *object_class = getWithHandle(class_handle);
 
-    if (SubOrUnsub == RTI_TRUE)
-        D.Out(pdInit, "Federate %d attempts to subscribe to Object Class %d.",
-              theFederateHandle, theClassHandle);
-    else
-        D.Out(pdTerm, "Federate %d attempts to unsubscribe from Object "
-              "Class %d.", theFederateHandle, theClassHandle);
+    bool need_discover = object_class->subscribe(federate, attributes, nb, region);
 
-    // It may throw AttributeNotDefined
-    Boolean result = theClass->subscribe(theFederateHandle,
-                                         theAttributeList,
-                                         theListSize,
-                                         SubOrUnsub);
-
-    // If Result is true, the Federate has never been a subscriber of this
-    // class, so it my have missed some DiscoverObject messages, for this
-    // class or one of its sub-class. Therefore, we must start a recursive
-    // process to check this class and its subclass for possible instances
-    // to discover, until we have reached an already subscribed class in
-    // each branch.
-
-    if (result == RTI_TRUE)
-        recursiveDiscovering(theClass->getHandle(), // Start process with this class
-                             theFederateHandle, // Send msgs to this Federate
-                             theClass->getHandle());
-    // Discovered objects belong to the same class, the original one.
+    if (need_discover)
+	object_class->recursiveDiscovering(federate, class_handle);
 }
 
 // ----------------------------------------------------------------------------
 //! updateAttributeValues.
 void
-ObjectClassSet::updateAttributeValues(FederateHandle theFederateHandle,
-                                      ObjectHandle theObjectHandle,
-                                      AttributeHandle *theAttribArray,
-                                      AttributeValue *theValueArray,
-                                      UShort theArraySize,
-                                      FederationTime theTime,
-                                      const char *theUserTag)
+ObjectClassSet::updateAttributeValues(FederateHandle federate,
+                                      ObjectHandle object_handle,
+                                      AttributeHandle *attributes,
+                                      AttributeValue *values,
+                                      UShort nb,
+                                      FederationTime time,
+                                      const char *tag)
     throw (ObjectNotKnown,
            AttributeNotDefined,
            AttributeNotOwned,
            RTIinternalError,
            InvalidObjectHandle)
 {
-    // It may throw ObjectNotKnown
-    ObjectClass * objectClass = getInstanceClass(theObjectHandle);
-    ObjectClassHandle currentClass = objectClass->getHandle();
+    Object *object = getObject(object_handle);
+    ObjectClass *object_class = getWithHandle(object->getClass());
+    ObjectClassHandle current_class = object_class->getHandle();
 
     D.Out(pdProtocol, "Federate %d Updating object %d from class %d.",
-          theFederateHandle, theObjectHandle, currentClass);
+          federate, object_handle, current_class);
 
     // It may throw a bunch of exceptions
     ObjectClassBroadcastList *ocbList = NULL ;
-    ocbList = objectClass->updateAttributeValues(theFederateHandle,
-                                                 theObjectHandle,
-                                                 theAttribArray,
-                                                 theValueArray,
-                                                 theArraySize,
-                                                 theTime,
-                                                 theUserTag);
+    ocbList = object_class->updateAttributeValues(
+	federate, object, attributes, values, nb, time, tag);
 
     // Broadcast ReflectAttributeValues message recursively
-    currentClass = objectClass->Father ;
+    current_class = object_class->getSuperclass();
 
-    while (currentClass != 0) {
+    while (current_class != 0) {
         D.Out(pdProtocol,
               "Broadcasting RAV msg to parent class %d for instance %d.",
-              currentClass, theObjectHandle);
+              current_class, object_handle);
 
         // It may throw ObjectClassNotDefined
-        objectClass = getWithHandle(currentClass);
-        objectClass->broadcastClassMessage(ocbList);
+        object_class = getWithHandle(current_class);
+        object_class->broadcastClassMessage(ocbList, object);
 
-        currentClass = objectClass->Father ;
+        current_class = object_class->getSuperclass();
     }
 
     delete ocbList ;
@@ -522,7 +481,7 @@ negotiatedAttributeOwnershipDivestiture(FederateHandle theFederateHandle,
                                                              theTag);
 
     // Broadcast ReflectAttributeValues message recursively
-    currentClass = objectClass->Father ;
+    currentClass = objectClass->getSuperclass();
 
     while (currentClass != 0) {
         D.Out(pdProtocol,
@@ -533,7 +492,7 @@ negotiatedAttributeOwnershipDivestiture(FederateHandle theFederateHandle,
         objectClass = getWithHandle(currentClass);
         objectClass->broadcastClassMessage(ocbList);
 
-        currentClass = objectClass->Father ;
+        currentClass = objectClass->getSuperclass();
     }
 
     delete ocbList ;
@@ -591,7 +550,7 @@ unconditionalAttributeOwnershipDivestiture(FederateHandle theFederateHandle,
                                                    theListSize);
 
     // Broadcast ReflectAttributeValues message recursively
-    currentClass = objectClass->Father ;
+    currentClass = objectClass->getSuperclass();
 
     while (currentClass != 0) {
         D.Out(pdProtocol,
@@ -602,7 +561,7 @@ unconditionalAttributeOwnershipDivestiture(FederateHandle theFederateHandle,
         objectClass = getWithHandle(currentClass);
         objectClass->broadcastClassMessage(ocbList);
 
-        currentClass = objectClass->Father ;
+        currentClass = objectClass->getSuperclass();
     }
 
     delete ocbList ;
@@ -683,4 +642,4 @@ cancelAttributeOwnershipAcquisition(FederateHandle theFederateHandle,
 
 } // namespace certi
 
-// $Id: ObjectClassSet.cc,v 3.14 2003/05/23 13:21:48 breholee Exp $
+// $Id: ObjectClassSet.cc,v 3.20 2005/04/30 17:16:08 breholee Exp $
