@@ -1,25 +1,20 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
-// This file is part of CERTI-libCERTI
-//
-// CERTI-libCERTI is free software ; you can redistribute it and/or
+// This program is free software ; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public License
 // as published by the Free Software Foundation ; either version 2 of
 // the License, or (at your option) any later version.
 //
-// CERTI-libCERTI is distributed in the hope that it will be useful, but
+// This program is distributed in the hope that it will be useful, but
 // WITHOUT ANY WARRANTY ; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 // Lesser General Public License for more details.
 //
 // You should have received a copy of the GNU Lesser General Public
 // License along with this program ; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA
-//
-// $Id: SocketMC.cc,v 3.8 2004/03/14 00:24:55 breholee Exp $
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -36,35 +31,32 @@ namespace certi {
 // ----------------------------------------------------------------------------
 SocketMC::SocketMC()
 {
-    _est_init_mc = RTI_FALSE ;
+    _est_init_mc = false ;
     _num_msg = 1 ;
 }
 
-
 // ----------------------------------------------------------------------------
 SocketMC::~SocketMC()
 {
-    if (_est_init_mc)
-        {
-            ::close(_socket_mc);
-            ::close(_socket_emetteur);
-            _est_init_mc = RTI_FALSE ;
-        }
+    if (_est_init_mc) {
+	::close(_socket_mc);
+	::close(_socket_emetteur);
+	_est_init_mc = false ;
+    }
 }
 
 
 // ----------------------------------------------------------------------------
-void SocketMC::CreerSocketMC(char *addr, unsigned long port)
+void
+SocketMC::CreerSocketMC(char *addr, unsigned long port)
 {
     assert(addr!=NULL);
-
     CreerSocketMC(inet_addr(addr), port);
 }
 
-
 // ----------------------------------------------------------------------------
-void SocketMC::
-CreerSocketMC(unsigned long addr, unsigned long port)
+void
+SocketMC::CreerSocketMC(unsigned long addr, unsigned long port)
 {
     struct ip_mreq _mreq ;
     unsigned long _mreqlen ;
@@ -74,11 +66,10 @@ CreerSocketMC(unsigned long addr, unsigned long port)
 
     // creation du socket recepteur
     _socket_mc = socket(AF_INET, SOCK_DGRAM, 0);
-    if (_socket_mc < 0)
-        {
-            perror("socket1");
-            exit(-1);
-        }
+    if (_socket_mc < 0) {
+	perror("socket1");
+	exit(-1);
+    }
 
     memset(&_sin, sizeof(_sin), sizeof(_sin));
     _sin.sin_family = AF_INET ;
@@ -86,34 +77,29 @@ CreerSocketMC(unsigned long addr, unsigned long port)
     _sin.sin_port = htons(port);
     _sinlen = sizeof(_sin);
 
-    if (bind(_socket_mc, (struct sockaddr *)&_sin, _sinlen) < 0)
-        {
-            perror("SocketMC: bind");
-            exit(-1);
-        }
+    if (bind(_socket_mc, (struct sockaddr *)&_sin, _sinlen) < 0) {
+	perror("SocketMC: bind");
+	exit(-1);
+    }
 
     // joindre le groupe multiCast
-
     _mreq.imr_multiaddr.s_addr = addr ;
     _mreq.imr_interface.s_addr = htonl(INADDR_ANY);
     _mreqlen = sizeof(_mreq);
     if (setsockopt(_socket_mc,
                    IPPROTO_IP,
                    IP_ADD_MEMBERSHIP,
-                   (char *)&_mreq, _mreqlen) < 0)
-        {
-            perror("setsockopt");
-            exit(-1);
-        }
+                   (char *)&_mreq, _mreqlen) < 0) {
+	perror("setsockopt");
+	exit(-1);
+    }
 
     // creation du socket emetteur
-
     _socket_emetteur = socket(AF_INET, SOCK_DGRAM, 0);
-    if (_socket_emetteur < 0)
-        {
-            perror("socket2");
-            exit(-1);
-        }
+    if (_socket_emetteur < 0) {
+	perror("socket2");
+	exit(-1);
+    }
 
     memset(&_sin_e, sizeof(_sin_e), sizeof(_sin_e));
     _sin_e.sin_family = AF_INET ;
@@ -122,21 +108,18 @@ CreerSocketMC(unsigned long addr, unsigned long port)
     _sinlen_e = sizeof(_sin_e);
 
     // l'initialisation de la communication multicast est reussie
-    _est_init_mc = RTI_TRUE ;
+    _est_init_mc = true ;
 }
 
-
-
 // ----------------------------------------------------------------------------
 void
-SocketMC::send(void *Buffer, unsigned long)
+SocketMC::send(const unsigned char *Buffer, size_t)
     throw (NetworkError, NetworkSignal)
 {
     // BUG:Revoir ca en faisant le contraire(EmettreMC appelle Emettre)
     sendMC((NetworkMessage*) Buffer);
 }
 
-
 // ----------------------------------------------------------------------------
 void
 SocketMC::receive(void *Buffer, unsigned long)
@@ -153,7 +136,6 @@ SocketMC::returnSocket() const
     return _socket_mc ;
 }
 
-
 // ----------------------------------------------------------------------------
 unsigned long
 SocketMC::returnAdress() const
@@ -162,19 +144,19 @@ SocketMC::returnAdress() const
 }
 
 // ----------------------------------------------------------------------------
-void SocketMC::close()
+void
+SocketMC::close()
 {
-    if (_est_init_mc)
-        {
-            ::close(_socket_mc);
-            ::close(_socket_emetteur);
-            _est_init_mc = RTI_FALSE ;
-        }
+    if (_est_init_mc) {
+	::close(_socket_mc);
+	::close(_socket_emetteur);
+	_est_init_mc = false ;
+    }
 }
 
-
 // ----------------------------------------------------------------------------
-void SocketMC::sendMC(NetworkMessage *message)
+void
+SocketMC::sendMC(NetworkMessage *message)
 {
     int cnt ;
 
@@ -186,18 +168,15 @@ void SocketMC::sendMC(NetworkMessage *message)
     cnt = sendto(_socket_emetteur, (char *)message, TAILLE_MSG_RESEAU, 0,
                  (struct sockaddr *)&_sin_e, _sinlen_e);
 
-
-    if (cnt < 0)
-        {
-            perror("Send");
-            exit(-1);
-        }
+    if (cnt < 0) {
+	perror("Send");
+	exit(-1);
+    }
 }
 
-
-
 // ----------------------------------------------------------------------------
-char *SocketMC::receiveMC(NetworkMessage *message)
+char *
+SocketMC::receiveMC(NetworkMessage *message)
 {
     int cnt ;
 
@@ -205,47 +184,40 @@ char *SocketMC::receiveMC(NetworkMessage *message)
 
     cnt = recvfrom(_socket_mc, (char *)message, TAILLE_MSG_RESEAU, 0,
                    (struct sockaddr *)&_sin, &_sinlen);
-    if (cnt < 0)
-        {
-            perror("Recv");
-            exit(1);
-        }
+    if (cnt < 0) {
+	perror("Recv");
+	exit(1);
+    }
 
     return(inet_ntoa(_sin.sin_addr));
 }
 
 // ----------------------------------------------------------------------------
+/** Wait for a socket event, until a time-out. The time-out is given
+    with seconds and microseconds in parameter.
+    @param sec seconds
+    @param usec microseconds
+    @return number of waiting events, 0 if time-out. May be negative
+    in case of error
+ */
 int SocketMC::timeoutMC(int sec, int usec)
 {
-    fd_set fdset ;
-    struct timeval timeout ;
-    int i ;
     assert(_est_init_mc);
 
+    struct timeval timeout ;
     timeout.tv_sec = sec ;
     timeout.tv_usec = usec ;
 
+    fd_set fdset ;
     FD_ZERO(&fdset);
     FD_SET(_socket_mc, &fdset);
-    while ((i = portableSelect(&fdset, &timeout) < 0) && (errno == EINTR))
-	;
-    return i ;
+
+    int nb = 0 ;
+    do {
+	nb = select(_socket_mc+1, SELECT_TYPE_ARG234 &fdset, NULL, NULL, &timeout);
+    } while (nb < 0 && errno == EINTR);
+
+    return nb ;
 }
 
-// ----------------------------------------------------------------------------
-/** Portable select
- */
-int
-SocketMC::portableSelect(fd_set *fdset, struct timeval *time_out)
-{
-#ifdef WITH_CYGWIN
-    return select(_socket_mc+1, SELECT_TYPE_ARG234 fdset, NULL, NULL, time_out);
-#else
-    return select(sysconf(_SC_OPEN_MAX), SELECT_TYPE_ARG234 fdset, NULL, NULL, time_out);
-#endif
-}
-
-}
-
-// EOF $Id: SocketMC.cc,v 3.8 2004/03/14 00:24:55 breholee Exp $
-
+} // namespace certi
