@@ -1,15 +1,15 @@
 // ----------------------------------------------------------------------------
 // CERTI - HLA RunTime Infrastructure
-// Copyright (C) 2002, 2003  ONERA
+// Copyright (C) 2002-2005  ONERA
 //
-// This file is part of CERTI
+// This file is part of Billard
 //
-// CERTI is free software ; you can redistribute it and/or modify
+// Billard is free software ; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation ; either version 2 of the License, or
 // (at your option) any later version.
 //
-// CERTI is distributed in the hope that it will be useful,
+// Billard is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY ; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 // GNU General Public License for more details.
@@ -17,40 +17,54 @@
 // You should have received a copy of the GNU General Public License
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
-//
-// $Id: Billard.cc,v 3.7 2004/03/04 20:19:05 breholee Exp $
 // ----------------------------------------------------------------------------
 
 #include "Billard.hh"
 
 #include <config.h>
-#include "Objects.hh"
 #include "PrettyDebug.hh"
 
 #include <unistd.h>
 
+using std::auto_ptr ;
 using std::string ;
 using std::endl ;
 using std::cout ;
+using std::vector ;
 
 static pdCDebug D("BILLARD", __FILE__);
 
+// ----------------------------------------------------------------------------
 /** Constructor
  */
 Billard::Billard(string federate_name)
-    : rtiamb(), objects(rtiamb, *this, 500, 100),
+    : rtiamb(),
+      local(0),
       federateName(federate_name),
-      handle(0), creator(false), nbTicks(0),
-      regulating(false), constrained(false), localTime(0.0), TIME_STEP(1.0),
-      XMAX(500), YMAX(100) { }
+      handle(0),
+      creator(false),
+      nbTicks(0),
+      regulating(false),
+      constrained(false),
+      localTime(0.0),
+      TIME_STEP(1.0),
+      XMAX(500),
+      YMAX(100)
+{
+}
 
+// ----------------------------------------------------------------------------
 /** Destructor
  */
-Billard::~Billard() { }
+Billard::~Billard()
+    throw (RTI::FederateInternalError)
+{
+}
 
+// ----------------------------------------------------------------------------
 /** Get the federate handle
  */
-FederateHandle
+RTI::FederateHandle
 Billard::getHandle() const
 {
     return handle ;
@@ -73,7 +87,7 @@ Billard::join(string federation_name, string fdd_name)
         D.Out(pdInit, "Federation execution created.");
         creator = true ;
     }
-    catch (FederationExecutionAlreadyExists& e) {
+    catch (RTI::FederationExecutionAlreadyExists& e) {
         D.Out(pdInit, "Federation execution already created.");
     }
 
@@ -90,18 +104,18 @@ Billard::join(string federation_name, string fdd_name)
             joined = true ;
             break ;
         }
-        catch (FederateAlreadyExecutionMember& e) {
+        catch (RTI::FederateAlreadyExecutionMember& e) {
             D[pdExcept] << "Federate " << federateName.c_str()
                         << "already exists." << endl ;
 
             throw ;
         }
-        catch (FederationExecutionDoesNotExist& e) {
+        catch (RTI::FederationExecutionDoesNotExist& e) {
             D.Out(pdExcept, "Federate %s : FederationExecutionDoesNotExist.",
                   federateName.c_str());
             // sleep(1);
         }
-        catch (Exception& e) {
+        catch (RTI::Exception& e) {
             D.Out(pdExcept,
                   "Federate %s :Join Federation Execution failed : %d .",
                   federateName.c_str(), &e);
@@ -122,7 +136,7 @@ Billard::pause()
             rtiamb.registerFederationSynchronizationPoint(
                 "Init", "Waiting all players.");
         }
-        catch (Exception& e) {
+        catch (RTI::Exception& e) {
             D[pdExcept] << "Federate " << federateName.c_str()
                         << " : Register Synchronization Point failed : %d"
                         << endl ;
@@ -174,7 +188,7 @@ Billard::setTimeRegulation(bool start_constrained, bool start_regulating)
                     regulating = true ;
                     break ;
                 }
-                catch (FederationTimeAlreadyPassed) {
+                catch (RTI::FederationTimeAlreadyPassed) {
                     // Si Je ne suis pas le premier, je vais les rattraper.
                     rtiamb.queryFederateTime(localTime);
 
@@ -187,13 +201,13 @@ Billard::setTimeRegulation(bool start_constrained, bool start_regulating)
                         try {
                             tick();
                         }
-                        catch (RTIinternalError) {
+                        catch (RTI::RTIinternalError) {
                             printf("RTIinternalError Raised in tick.\n");
                             exit(-1);
                         }
                     }
                 }
-                catch (RTIinternalError) {
+                catch (RTI::RTIinternalError) {
                     printf("RTIinternalError Raised in setTimeRegulating.\n");
                     exit(-1);
                 }
@@ -239,7 +253,7 @@ Billard::synchronize(int autostart)
                 D.Out(pdInit, "not paused");
                 tick();
             }
-            catch (Exception& e) {
+            catch (RTI::Exception& e) {
                 D.Out(pdExcept, "******** Exception ticking the RTI : %d ", &e);
             }
         D.Out(pdDebug, "paused");
@@ -247,7 +261,7 @@ Billard::synchronize(int autostart)
         try {
             rtiamb.synchronizationPointAchieved("Init");
         }
-        catch (Exception& e) {
+        catch (RTI::Exception& e) {
             D.Out(pdExcept, "**** Exception achieving a synchronization "
                   "point by creator : %d", &e);
         }
@@ -256,7 +270,7 @@ Billard::synchronize(int autostart)
             try {
                 tick();
             }
-            catch (Exception& e) {
+            catch (RTI::Exception& e) {
                 D.Out(pdExcept, "**** Exception ticking the RTI : %d.", &e);
             }
     }
@@ -273,7 +287,7 @@ Billard::synchronize(int autostart)
                 try {
                     tick();
                 }
-                catch (Exception& e) {
+                catch (RTI::Exception& e) {
                     D.Out(pdExcept,
                           "******** Exception ticking the RTI : %d.", &e);
                 }
@@ -286,7 +300,7 @@ Billard::synchronize(int autostart)
             rtiamb.synchronizationPointAchieved("Init");
             D.Out(pdInit, "Pause achieved.");
         }
-        catch (Exception& e) {
+        catch (RTI::Exception& e) {
             D.Out(pdExcept,
                   "**** Exception achieving a synchronization point : %d",
                   &e);
@@ -298,7 +312,7 @@ Billard::synchronize(int autostart)
             try {
                 tick();
             }
-            catch (Exception& e) {
+            catch (RTI::Exception& e) {
                 D.Out(pdExcept, "******** Exception ticking the RTI : %d.", &e);
             }
         }
@@ -315,7 +329,7 @@ Billard::synchronize(int autostart)
 void
 Billard::init(int seed)
 {
-    objects.init(seed);
+    local.init(seed);
 }
 
 // ----------------------------------------------------------------------------
@@ -326,7 +340,7 @@ Billard::init(int seed)
 void
 Billard::init(int x, int y)
 {
-    objects.init(x, y);
+    local.init(x, y);
 }
 
 // ----------------------------------------------------------------------------
@@ -335,8 +349,7 @@ Billard::init(int x, int y)
 void
 Billard::declare()
 {
-    // only objects in this class
-    objects.declare(federateName);
+    local.ID = registerBallInstance(federateName.c_str());
 }
 
 // ----------------------------------------------------------------------------
@@ -358,22 +371,26 @@ Billard::step()
 	granted = false ;
         rtiamb.timeAdvanceRequest(time_aux);
     }
-    catch (Exception& e) {
+    catch (RTI::Exception& e) {
         D.Out(pdExcept, "******* Exception sur timeAdvanceRequest.");
     }
 
+    for (vector<Ball>::iterator it = remote.begin(); it != remote.end(); ++it) {
+	it->active = false ;
+    }	
+
     while (!granted) {
         try {
             tick();
         }
-        catch (Exception& e) {
+        catch (RTI::Exception& e) {
             D.Out(pdExcept, "******** Exception ticking the RTI : %d.", &e);
         }
     }
     try {
         rtiamb.queryFederateTime(localTime);
     }
-    catch (Exception& e) {
+    catch (RTI::Exception& e) {
         D.Out(pdExcept,
               "**** Exception asking for federate local time : ", &e);
     }
@@ -383,10 +400,49 @@ Billard::step()
 
     RTIfedTime next_step(localTime + TIME_STEP);
 
-    objects.erase();
-    objects.step(next_step);
-    objects.display();
-    objects.update(next_step);
+    local.erase();
+
+    vector<Ball>::iterator it ;
+
+    for (it = remote.begin(); it != remote.end(); ++it) {
+        if (it->ID != 0 && it->active && local.checkBallCollision(&(*it))) {
+            sendInteraction(local.dx, local.dy, next_step, it->ID);
+            // On prend la vitesse de l'autre sauf dans le cas ou
+            // on avait deja la meme. Dans ce cas, on inverse la notre.
+            if ((local.dx == it->dx) && (local.dy == it->dy)) {
+                local.dx = -local.dx ;
+                local.dy = -local.dy ;
+            }
+            else
+                local.setDirection(it->dx, it->dy);
+        }
+        it->display();
+        D.Out(pdTrace, "no collision.");
+    }
+
+    // Teste la collision avec le bord
+    D.Out(pdTrace, "Border collisions...");
+    local.checkBorderCollision(XMAX, YMAX);
+
+    local.x += local.dx ;
+    local.y += local.dy ;
+
+    checkRegions();
+
+    local.display();
+
+    sendUpdate(local.x, local.y, (int) local.color, next_step, local.ID);
+
+    D.Out(pdTrace, "End of step");
+}
+
+// ----------------------------------------------------------------------------
+/** Check DDM regions
+ */
+void
+Billard::checkRegions()
+{
+    // Nothing, in this default demo
 }
 
 // ----------------------------------------------------------------------------
@@ -395,17 +451,24 @@ Billard::step()
 void
 Billard::resign()
 {
-    objects.destroy(localTime);
+    try {
+        rtiamb.deleteObjectInstance(local.ID, localTime, "DO");
+        D.Out(pdRegister, "Local object deleted from federation.");
+    }
+    catch (RTI::Exception &e) {
+        D.Out(pdExcept, "**** Exception delete object : %d", &e);
+    }
+
     D.Out(pdTerm, "Local objects deleted.");
 
     setTimeRegulation(false, false);
 
     try {
         rtiamb.resignFederationExecution(
-            DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES);
+            RTI::DELETE_OBJECTS_AND_RELEASE_ATTRIBUTES);
         D.Out(pdTerm, "Just resigned from federation");
     }
-    catch (Exception &e) {
+    catch (RTI::Exception &e) {
         D.Out(pdExcept,
               "** Exception during resignFederationExecution by federate");
     }
@@ -421,7 +484,7 @@ Billard::resign()
                 D.Out(pdTerm, "Federation destruction granted.");
                 break ;
             }
-            catch (FederatesCurrentlyJoined) {
+            catch (RTI::FederatesCurrentlyJoined) {
                 sleep(5);
             }
         }
@@ -440,26 +503,24 @@ Billard::publishAndSubscribe()
     getHandles();
 
     // Add PositionX et PositionY to the attribute set
-    AttributeHandleSet *AttributeSet = AttributeHandleSetFactory::create(3);
-    AttributeSet->add(AttrXID);
-    AttributeSet->add(AttrYID);
+    auto_ptr<RTI::AttributeHandleSet> attributes(RTI::AttributeHandleSetFactory::create(3));
+    attributes->add(AttrXID);
+    attributes->add(AttrYID);
 
     // Subscribe to Bille objects.
     D[pdDebug] << "subscribe: class " << BilleClassID << ", attributes "
 	       << AttrXID << " and " << AttrYID << "... " ;
-    rtiamb.subscribeObjectClassAttributes(BilleClassID, *AttributeSet, RTI_TRUE);
+    rtiamb.subscribeObjectClassAttributes(BilleClassID, *attributes, RTI::RTI_TRUE);
     D[pdDebug] << "done." << endl ;
 
     // Publish Boule Objects.
-    AttributeSet->add(AttrColorID);
-    rtiamb.publishObjectClass(BouleClassID, *AttributeSet);
+    attributes->add(AttrColorID);
+    rtiamb.publishObjectClass(BouleClassID, *attributes);
 
     // Publish and subscribe to Bing interactions
-    rtiamb.subscribeInteractionClass(BingClassID, RTI_TRUE);
+    rtiamb.subscribeInteractionClass(BingClassID, RTI::RTI_TRUE);
     rtiamb.publishInteractionClass(BingClassID);
 
-    AttributeSet->empty();
-
     D.Out(pdInit, "Local Objects and Interactions published and subscribed.");
 }
 
@@ -497,13 +558,13 @@ Billard::getHandles()
   InteractionTime.
 */
 void
-Billard::sendInteraction(double dx, double dy, const FedTime& InteractionTime,
-                     ObjectHandle id)
+Billard::sendInteraction(double dx, double dy, const RTI::FedTime& InteractionTime,
+			 RTI::ObjectHandle id)
 {
     char buf[512] ;
-    ParameterHandleValuePairSet *parameterSet=NULL ;
+    RTI::ParameterHandleValuePairSet *parameterSet=NULL ;
 
-    parameterSet = ParameterSetFactory::create(3);
+    parameterSet = RTI::ParameterSetFactory::create(3);
 
     sprintf(buf, "%ld", id);
     parameterSet->add(ParamBoulID, buf, strlen(buf)+1);
@@ -531,7 +592,7 @@ Billard::sendInteraction(double dx, double dy, const FedTime& InteractionTime,
     try {
         rtiamb.sendInteraction(BingClassID, *parameterSet, InteractionTime, "");
     }
-    catch (Exception& e) {
+    catch (RTI::Exception& e) {
         D.Out(pdExcept, "**** Exception sending interaction : %d", &e);
     }
 
@@ -547,13 +608,13 @@ Billard::sendInteraction(double dx, double dy, const FedTime& InteractionTime,
     \param id Object handle (ball)
  */
 void
-Billard::sendUpdate(double x, double y, int color, const FedTime& UpdateTime,
-                ObjectHandle id)
+Billard::sendUpdate(double x, double y, int color, const RTI::FedTime& UpdateTime,
+		    RTI::ObjectHandle id)
 {
     char buf[512] ;
-    AttributeHandleValuePairSet *attributeSet ;
+    RTI::AttributeHandleValuePairSet *attributeSet ;
 
-    attributeSet = AttributeSetFactory::create(3);
+    attributeSet = RTI::AttributeSetFactory::create(3);
 
     D.Out(pdTrace, "SendUpdate.");
 
@@ -578,7 +639,7 @@ Billard::sendUpdate(double x, double y, int color, const FedTime& UpdateTime,
         // logfile << string(((RTIfedTime) UpdateTime).getTime()) << " : UAV "
         // << string(Local.x) << " " << string(Local.y) << endl ;
     }
-    catch (Exception& e) {
+    catch (RTI::Exception& e) {
         D.Out(pdExcept, "**** Exception updating attribute values: %d", &e);
     }
 
@@ -590,7 +651,7 @@ Billard::sendUpdate(double x, double y, int color, const FedTime& UpdateTime,
     \param s Object name to provide to the RTI
     \return The created object handle
  */
-ObjectHandle
+RTI::ObjectHandle
 Billard::registerBallInstance(const char *s)
 {
     return rtiamb.registerObjectInstance(BouleClassID, s);
@@ -604,19 +665,28 @@ Billard::registerBallInstance(const char *s)
 /** Callback : discover object instance
  */
 void
-Billard::discoverObjectInstance(ObjectHandle theObject,
-                            ObjectClassHandle theObjectClass,
-                            const char */*theObjectName*/)
-    throw (CouldNotDiscover,
-           ObjectClassNotKnown,
-           InvalidFederationTime,
-           FederateInternalError)
+Billard::discoverObjectInstance(RTI::ObjectHandle theObject,
+				RTI::ObjectClassHandle theObjectClass,
+				const char */*theObjectName*/)
+    throw (RTI::CouldNotDiscover, RTI::ObjectClassNotKnown, 
+	   RTI::FederateInternalError)
 {
     if (theObjectClass != BilleClassID) {
         D.Out(pdError, "Object of Unknown Class discovered.");
-        throw RTIinternalError();
+        throw RTI::FederateInternalError("");
+    }
+    
+    bool already = false ;
+    for (vector<Ball>::iterator i = remote.begin(); i != remote.end(); ++i) {
+	if (i->ID == theObject) {
+	    cout << "Rediscovered object " << theObject << endl ;
+	    already = true ;
+	}
+    }
+    if (!already) {
+	cout << "Discovered object " << theObject << endl ;
+	remote.push_back(Ball(theObject));
     }
-    objects.discover(theObject);
 }
 
 // ----------------------------------------------------------------------------
@@ -624,10 +694,10 @@ Billard::discoverObjectInstance(ObjectHandle theObject,
  */
 void
 Billard::announceSynchronizationPoint(const char *label, const char */*tag*/)
-    throw (FederateInternalError)
+    throw (RTI::FederateInternalError)
 {
     if (strcmp(label, "Init") == 0) {
-        paused = RTI_TRUE ;
+        paused = true ;
         D.Out(pdProtocol, "announceSynchronizationPoint.");
     }
     else {
@@ -641,7 +711,7 @@ Billard::announceSynchronizationPoint(const char *label, const char */*tag*/)
  */
 void
 Billard::federationSynchronized(const char *label)
-    throw (FederateInternalError)
+    throw (RTI::FederateInternalError)
 {
     if (strcmp(label, "Init") == 0) {
         paused = false ;
@@ -654,21 +724,21 @@ Billard::federationSynchronized(const char *label)
 /** Callback : receive interaction
  */
 void
-Billard::receiveInteraction(InteractionClassHandle theInteraction,
-                        const ParameterHandleValuePairSet& theParameters,
-                        const FedTime& /*theTime*/,
+Billard::receiveInteraction(RTI::InteractionClassHandle theInteraction,
+                        const RTI::ParameterHandleValuePairSet& theParameters,
+                        const RTI::FedTime& /*theTime*/,
                         const char */*theTag*/,
-                        EventRetractionHandle /*theHandle*/)
-    throw (InteractionClassNotKnown,
-           InteractionParameterNotKnown,
-           InvalidFederationTime,
-           FederateInternalError)
+                        RTI::EventRetractionHandle /*theHandle*/)
+    throw (RTI::InteractionClassNotKnown,
+           RTI::InteractionParameterNotKnown,
+           RTI::InvalidFederationTime,
+           RTI::FederateInternalError)
 {
     char *parmValue ;
-    ULong valueLength ;
+    RTI::ULong valueLength ;
     int dx1 = 0 ;
     int dy1 = 0 ;
-    ObjectHandle h1 = 0 ;
+    RTI::ObjectHandle h1 = 0 ;
     bool bille = false ;
 
     D.Out(pdTrace, "Fed : receiveInteraction");
@@ -682,7 +752,7 @@ Billard::receiveInteraction(InteractionClassHandle theInteraction,
           theParameters.size());
 
     for (unsigned int j = 0 ; j < theParameters.size(); ++j) {
-        ParameterHandle handle = theParameters.getHandle(j);
+        RTI::ParameterHandle handle = theParameters.getHandle(j);
 
         valueLength = theParameters.getValueLength(j);
         parmValue = new char[valueLength] ;
@@ -720,7 +790,10 @@ Billard::receiveInteraction(InteractionClassHandle theInteraction,
                 }
     }
     if (bille) {
-        objects.receive(h1, dx1, dy1);
+	if (h1 == local.ID) {
+	    local.dx = dx1 ;
+	    local.dy = dy1 ;
+	}
     }
 }
 
@@ -729,32 +802,30 @@ Billard::receiveInteraction(InteractionClassHandle theInteraction,
  */
 void
 Billard::reflectAttributeValues(
-    ObjectHandle theObject,
-    const AttributeHandleValuePairSet& theAttributes,
-    const FedTime& /*theTime*/,
+    RTI::ObjectHandle theObject,
+    const RTI::AttributeHandleValuePairSet& theAttributes,
+    const RTI::FedTime& /*theTime*/,
     const char */*theTag*/,
-    EventRetractionHandle /*theHandle*/)
-    throw (ObjectNotKnown,
-           AttributeNotKnown,
-           InvalidFederationTime,
-           FederateInternalError)
+    RTI::EventRetractionHandle /*theHandle*/)
+    throw (RTI::ObjectNotKnown,
+           RTI::AttributeNotKnown,
+           RTI::InvalidFederationTime,
+           RTI::FederateInternalError)
 {
     D.Out(pdTrace, "reflectAttributeValues");
 
     float x1 = 0 ;
     float y1 = 0 ;
 
-    ULong valueLength ;
+    RTI::ULong valueLength ;
     char *attrValue ;
 
-
     D.Out(pdDebug, "reflectAttributeValues - nb attributs= %d",
           theAttributes.size());
 
-
     for (unsigned int j=0 ; j<theAttributes.size(); j++) {
 
-        AttributeHandle handle = theAttributes.getHandle(j);
+        RTI::AttributeHandle handle = theAttributes.getHandle(j);
         valueLength = theAttributes.getValueLength(j);
         attrValue = new char[valueLength] ;
         theAttributes.getValue(j, attrValue, valueLength);
@@ -779,31 +850,55 @@ Billard::reflectAttributeValues(
             D.Out(pdError, "Fed: ERREUR: handle inconnu.");
     }
 
-    objects.reflect(theObject, (int) x1, (int) y1);
+    vector<Ball>::iterator it = remote.begin() ;
+    while (it != remote.end() && it->ID != theObject)
+	++it ;
+
+    if (it == remote.end())
+        D.Out(pdError, "Fed: error, id not found (%d).", theObject);
+    else {
+        it->erase();
+
+        float oldx = it->x ;
+        it->x = x1 ;
+        it->dx = it->x - oldx ;
+
+        float oldy = it->y ;
+        it->y = y1 ;
+        it->dy = it->y - oldy ;
+
+	it->active = true ;
+    }
 }
 
 // ----------------------------------------------------------------------------
 /** Callback : remove object instance
  */
 void
-Billard::removeObjectInstance(ObjectHandle theObject,
-                          const FedTime &,
-                          const char *,
-                          EventRetractionHandle)
-    throw (ObjectNotKnown, InvalidFederationTime, FederateInternalError)
+Billard::removeObjectInstance(RTI::ObjectHandle theObject,
+			      const RTI::FedTime &,
+			      const char *,
+			      RTI::EventRetractionHandle)
+    throw (RTI::ObjectNotKnown, RTI::InvalidFederationTime, RTI::FederateInternalError)
 {
-    objects.remove(theObject);
+    vector<Ball>::iterator it ;
+
+    for (it = remote.begin(); it != remote.end(); ++it) {
+        if (it->ID == theObject) {
+            it->erase();
+            remote.erase(it);
+            return ;
+        }
+    }
 }
 
 // ----------------------------------------------------------------------------
 /** Callback : time advance granted
  */
 void
-Billard::timeAdvanceGrant(const FedTime& /*theTime*/)
-    throw (InvalidFederationTime, TimeAdvanceWasNotInProgress,
-           FederationTimeAlreadyPassed, FederateInternalError)
+Billard::timeAdvanceGrant(const RTI::FedTime& /*theTime*/)
+    throw (RTI::InvalidFederationTime, RTI::TimeAdvanceWasNotInProgress, 
+	   RTI::FederateInternalError)
 {    
     granted = true ;
 }
-
-// $Id: Billard.cc,v 3.7 2004/03/04 20:19:05 breholee Exp $
